from flask import Flask, Blueprint, Response

from . import discoverer_devices, discoverer_media, player

bp = Blueprint('endpoints', __name__)

@bp.route("/discover/chromecasts")
def discover_chromecasts():
    services = discoverer_devices.discover_chromecasts()
    chromecasts = []

    for chromecast in services or []:
        chromecasts.append({
            "ip": chromecast.host,
            "port": chromecast.port,
            "uuid": str(chromecast.uuid),
            "type": chromecast.model_name,
            "name": chromecast.friendly_name
        })

    return {"devices": chromecasts}

@bp.route("/discover/media")
def discover_media():
    return {"inputs": discoverer_media.discover_inputs()}

@bp.route("/player/cast/<input>/<ip>/<port>")
def cast(input, ip, port):
    player.play_media_to_chromecast(input, ip, port)
    return Response(status=200)

@bp.route("/player/play")
def play():
    player.play()
    return Response(status=200)

@bp.route("/player/pause")
def pause():
    player.pause()
    return Response(status=200)

@bp.route("/player/stop")
def stop():
    player.stop()
    return Response(status=200)


import sounddevice as sd

def discover_inputs():
    devices = sd.query_devices()
    input_devices = [device for device in devices if device['max_input_channels'] > 0]
    input_names = list(set(map(lambda d: d['name'], input_devices)))
    return input_names



import os
import re
from os.path import expanduser

home = expanduser("~")
app_data = ".audio_cast"
app_data_dir = f"{home}/{app_data}"

def isWindows():
    return os.name == 'nt'

def readFile(name):
    return open(f"{app_data_dir}/{name}", "r")

def writeFile(name):
    return open(f"{app_data_dir}/{name}", "w")

def createAppDir():
    os.makedirs(app_data_dir, exist_ok=True)

def get_alsa_input(input):
    pattern = '\((.+?)\)'
    try:
        found = input
        for match in re.findall(pattern, input):
            found = match
            if match.startswith('hw'):
                break
    except AttributeError:
        # Nothing found
        found = input
    return found



import click
from audio_cast import discoverer_devices, discoverer_media, player
from prettytable import PrettyTable

@click.group()
def cli():
    """Application to cast audio from an audio input to a chromecast device/group."""

@cli.command()
def discover_chromecasts():
    discoverer_devices.discover_chromecasts()

@cli.command()
def discover_media():
    inputs = discoverer_media.discover_inputs()
    table = PrettyTable()
    table.field_names = ["Input name"]
    for input in inputs:
        table.add_row([input])
    click.echo(table)

@cli.command()
@click.option('--i', required=True, type=str, help='The input device name to cast. Use discover_media if needed.')
@click.option('--c', required=True, type=str, help='The ip of the chromecast. Use discover_chromecasts if needed.')
@click.option('--p', required=True, type=int, help='The port of the chromecast.')
def cast(i, c, p):
    player.play_media_to_chromecast(i, c, p)
    click.echo(f"Casting {i} to {c} on port {p}.")
    click.echo("It may take a couple of seconds until the sound can be heard due to buffering.")

@cli.command()
def stop():
    if player.stop():
        click.echo("Stopping")
    else:
        click.echo("Nothing playing")


import vlc

from . import utils

_instance: vlc.Instance = None
player: vlc.MediaPlayer = None

def play_media_to_chromecast(input, chromecast_ip, port):
    m: vlc.Media
    if utils.isWindows(): # Windows path uses dshow
        m = instance().media_new_location("dshow://")
        m.add_option(f":adev={input}")
        m.add_option(":dshow-vdev=None")
    else: # Linux path uses alsa
        alsa_input = utils.get_alsa_input(input)
        m = instance().media_new_location(f"alsa://plug{alsa_input}")
    m.add_option(f":sout=#chromecast{{ip={chromecast_ip}, port={port}}}")
    m.add_option(f":demux-filter=demux_chromecast")

    stop()
    global player
    player = m.player_new_from_media()
    player.play()

def pause():
    if player:
        player.pause()

def play():
    if player:
        player.play()

def stop():
    global player
    if player:
        player.stop()
        player.release()
        player = None
        return True
    else:
        return False

def instance():
    global _instance
    if _instance is None:
        # _instance = vlc.Instance("--verbose 9")
        _instance = vlc.Instance()
    return _instance
